from django import forms
from django.forms.widgets import Select, SelectMultiple


class JazzminSelect(Select):
    template_name = "jazzmin/widgets/select.html"

    @property
    def media(self):
        return forms.Media(
            css={"all": ("adminlte/plugins/select2/select2.min.css",)}, js=("adminlte/plugins/select2/select2.min.js",)
        )


class JazzminSelectMultiple(SelectMultiple):
    template_name = "jazzmin/widgets/select.html"

    def build_attrs(self, base_attrs, extra_attrs=None):
        extra_attrs['multiple'] = 'multiple'
        return {**base_attrs, **(extra_attrs or {})}

    @property
    def media(self):
        return forms.Media(
            css={"all": ("adminlte/plugins/select2/select2.min.css",)}, js=("adminlte/plugins/select2/select2.min.js",)
        )



import copy
import itertools
import logging
import urllib.parse

from django.contrib.admin.views.main import PAGE_VAR
from django.contrib.auth import get_user_model
from django.template import Library
from django.template.loader import get_template
from django.templatetags.static import static
from django.utils.html import format_html
from django.utils.safestring import mark_safe

from .. import version
from ..settings import get_settings
from ..utils import order_with_respect_to, get_filter_id, get_custom_url, get_admin_url, get_model_permissions

User = get_user_model()
register = Library()
logger = logging.getLogger(__name__)
OPTIONS = get_settings()


@register.simple_tag(takes_context=True)
def get_side_menu(context):
    """
    Get the list of apps and models to render out in the side menu and on the dashboard page
    """
    user = context.get('user')
    if not user:
        return []

    model_permissions = get_model_permissions(user)

    menu = []
    available_apps = copy.deepcopy(context.get('available_apps', []))
    for app in available_apps:
        app_label = app['app_label'].lower()
        if app_label in OPTIONS['hide_apps']:
            continue

        allowed_models = []
        for model in app.get('models', []):
            model_str = '{app_label}.{model}'.format(app_label=app_label, model=model["object_name"]).lower()
            if model_str not in model_permissions:
                continue
            if model_str in OPTIONS.get('hide_models', []):
                continue

            model['icon'] = OPTIONS.get('icons', {}).get(model_str)
            allowed_models.append(model)

        for custom_link in OPTIONS.get('custom_links', {}).get(app_label, []):

            perm_matches = []
            for perm in custom_link.get('permissions', []):
                perm_matches.append(user.has_perm(perm))

            if not all(perm_matches):
                continue

            allowed_models.append({
                'custom': True,
                'name': custom_link.get('name'),
                'admin_url': get_custom_url(custom_link.get('url')),
                'icon': custom_link.get('icon'),
            })

        if len(allowed_models):
            app['models'] = allowed_models
            menu.append(app)

    if OPTIONS.get('order_with_respect_to'):
        menu = order_with_respect_to(menu, OPTIONS['order_with_respect_to'])

    return menu


@register.simple_tag
def get_top_menu(user):
    if not user:
        return []

    model_permissions = get_model_permissions(user)

    menu = []
    for item in get_settings().get('topmenu_links', []):

        perm_matches = []
        for perm in item.get('permissions', []):
            perm_matches.append(user.has_perm(perm))

        if not all(perm_matches):
            continue

        if item.get('model') and item.get('model').lower() not in model_permissions:
            continue

        if item.get('app'):
            item['app_children'] = list(filter(lambda x: x['model'] in model_permissions, item['app_children']))
            if len(item['app_children']) == 0:
                continue

        menu.append(item)

    return menu


@register.simple_tag
def get_jazzmin_settings():
    """
    Return Jazzmin settings
    """
    return OPTIONS


@register.simple_tag
def get_jazzmin_version():
    """
    Get the version for this package
    """
    return version


@register.simple_tag
def get_user_avatar(user):
    """
    For the given user, try to get the avatar image
    """
    no_avatar = static("adminlte/img/user2-160x160.jpg")

    if not OPTIONS.get('user_avatar'):
        return no_avatar

    avatar_field = getattr(user, OPTIONS['user_avatar'], None)
    if avatar_field:
        return avatar_field.url

    return no_avatar


@register.simple_tag
def jazzmin_paginator_number(cl, i):
    """
    Generate an individual page index link in a paginated list.
    """
    if i == '.':
        return format_html(
            '<li class="page-item">'
            '<a class="page-link" href="javascript:void(0);" data-dt-idx="3" tabindex="0">… </a>'
            '</li>'
        )

    elif i == cl.page_num:
        return format_html(("""
            <li class="page-item active">
            <a class="page-link" href="javascript:void(0);" data-dt-idx="3" tabindex="0">{num}
            </a>
            </li>
        """.format(num=i + 1)))

    else:
        query_string = cl.get_query_string({PAGE_VAR: i})
        end = mark_safe('end' if i == cl.paginator.num_pages - 1 else '')
        return format_html(("""
            <li class="page-item">
            <a href="{query_string}" class="page-link {end}" data-dt-idx="3" tabindex="0">{num}</a>
            </li>
        """).format(num=i + 1, query_string=query_string, end=end))


@register.simple_tag
def admin_extra_filters(cl):
    """
    Return the dict of used filters which is not included in list_filters form
    """
    used_parameters = list(itertools.chain(*(s.used_parameters.keys() for s in cl.filter_specs)))
    return dict((k, v) for k, v in cl.params.items() if k not in used_parameters)


@register.simple_tag
def jazzmin_list_filter(cl, spec):
    tpl = get_template(spec.template)
    choices = list(spec.choices(cl))
    field_key = get_filter_id(spec)
    matched_key = field_key
    for choice in choices:
        query_string = choice['query_string'][1:]
        query_parts = urllib.parse.parse_qs(query_string)

        value = ''
        matches = {}
        for key in query_parts.keys():
            if key == field_key:
                value = query_parts[key][0]
                matched_key = key
            elif key.startswith(field_key + '__') or '__' + field_key + '__' in key:
                value = query_parts[key][0]
                matched_key = key

            if value:
                matches[matched_key] = value

        # Iterate matches, use first as actual values, additional for hidden
        i = 0
        for key, value in matches.items():
            if i == 0:
                choice['name'] = key
                choice['value'] = value
            i += 1

    return tpl.render({'field_name': field_key, 'title': spec.title, 'choices': choices, 'spec': spec, })


@register.filter
def jazzy_admin_url(value):
    """
    Get the admin url for a given object
    """
    return get_admin_url(value)


@register.filter
def debug(value):
    """
    Add in a breakpoint here and use filter in templates for debugging ;)
    """
    return type(value)


@register.simple_tag
def sidebar_status(request):
    """
    Check if our sidebar is open or closed
    """
    if request.COOKIES.get('jazzy_menu', '') == 'closed':
        return 'sidebar-collapse'
    return ''


@register.filter
def can_view_self(perms):
    view_perm = '{}.view_{}'.format(User._meta.app_label, User._meta.model_name)
    change_perm = '{}.change_{}'.format(User._meta.app_label, User._meta.model_name)

    return perms[User._meta.app_label][view_perm] or perms[User._meta.app_label][change_perm]


@register.simple_tag
def header_class(header, forloop):
    classes = []
    sorted, asc, desc = header.get('sorted'), header.get('ascending'), header.get('descending')

    if forloop['counter0'] == 0:
        classes.append("djn-checkbox-select-all")

    if not header['sortable']:
        return ' '.join(classes)

    if sorted and asc:
        classes.append("sorting_asc")
    elif sorted and desc:
        classes.append("sorting_desc")
    else:
        classes.append("sorting")

    return ' '.join(classes)



import logging
from urllib.parse import urlencode

from django.apps import apps
from django.db.models.base import ModelBase

from jazzmin.compat import reverse, NoReverseMatch

logger = logging.getLogger(__name__)


def order_with_respect_to(first, reference):
    ranking = []
    max_num = len(first)

    for item in first:
        try:
            pos = reference.index(item['app_label'])
        except ValueError:
            pos = max_num

        ranking.append(pos)

    return [y for x, y in sorted(zip(ranking, first), key=lambda x: x[0])]


def get_admin_url(instance, **kwargs):
    """
    Return the admin URL for the given instance, model class or <app>/<model> string
    """
    url = '#'

    try:

        if type(instance) == str:
            app_label, model_name = instance.lower().split('.')
            url = reverse('admin:{app_label}_{model_name}_changelist'.format(
                app_label=app_label, model_name=model_name
            ))

        # Model class
        elif instance.__class__ == ModelBase:
            app_label, model_name = instance._meta.app_label, instance._meta.model_name
            url = reverse("admin:{app_label}_{model_name}_changelist".format(
                app_label=app_label, model_name=model_name
            ))

        # Model instance
        elif instance.__class__.__class__ == ModelBase and isinstance(instance, instance.__class__):
            app_label, model_name = instance._meta.app_label, instance._meta.model_name
            url = reverse("admin:{app_label}_{model_name}_change".format(
                app_label=app_label, model_name=model_name
            ), args=(instance.pk,))

    except NoReverseMatch:
        logger.error('Couldnt reverse url from {instance}'.format(instance=instance))

    if kwargs:
        url += '?{params}'.format(params=urlencode(kwargs))

    return url


def get_filter_id(spec):
    return getattr(spec, 'field_path', getattr(spec, 'parameter_name', spec.title))


def get_custom_url(url):
    """
    Take in a custom url, and try to reverse it
    """
    if not url:
        logger.warning('No url supplied in custom link')
        return '#'

    if '/' in url:
        return url
    try:
        url = reverse(url)
    except NoReverseMatch:
        logger.warning('Couldnt reverse {url}'.format(url=url))
        url = '#' + url

    return url


def get_model_meta(model_str):
    """
    Get the plural name
    """
    app, model = model_str.split('.')
    Model = apps.get_registered_model(app, model)
    return Model._meta


def get_app_admin_urls(app):
    """
    For the given app string, get links to all the app models admin views
    """
    if app not in apps.app_configs:
        logger.warning('{app} not found when generating links'.format(app=app))
        return []

    models = []
    for model in apps.app_configs[app].get_models():
        url = get_admin_url(model)

        # We have no admin class
        if url == '#':
            continue

        models.append({
            'url': get_admin_url(model),
            'model': '{app}.{model}'.format(app=model._meta.app_label, model=model._meta.model_name),
            'name': model._meta.verbose_name_plural.title()
        })

    return models


def get_model_permissions(user):
    """
    Create model permissions from the users permissions,

    e.g having any of auth.view_user, auth.change_user, auth.delete_user will grant you auth.user
    """
    permissions = set()
    for permission in user.get_all_permissions():
        app_label, model = permission.split('.')
        model = model.split('_')[1]
        permissions.add('{app_label}.{model}'.format(app_label=app_label, model=model))

    return permissions



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

from jazzmin import version

setup(
    name='django-jazzmin',
    version=version,
    url='https://github.com/farridav/django-jazzmin',
    description='Django admin theme based adminlte & bootstrap',
    long_description=open('README.rst').read(),
    author='farridav',
    maintainer='farridav',
    license='MIT',
    packages=find_packages(exclude=('tests', 'tests.*')),
    include_package_data=True,
    zip_safe=False,
    entry_points={},
    install_requires=[
        'django>=2',
    ]
)



import copy

from django.conf import settings
from django.contrib.admin import AdminSite

from .utils import get_admin_url, get_custom_url, get_model_meta, get_app_admin_urls

DEFAULT_SETTINGS = {
    # title of the window
    'site_title': AdminSite.site_title,

    # Title on the login screen
    'site_header': AdminSite.site_header,

    # Relative path to logo for your site, used for favicon and brand on top left (must be present in static files)
    'site_logo': 'adminlte/img/AdminLTELogo.png',

    # Welcome text on the login screen
    'welcome_sign': 'Welcome',

    # Copyright on the footer
    'copyright': '',

    # The model admin to search from the search bar, search bar omitted if excluded
    'search_model': None,

    # Field name on user model that contains avatar image
    'user_avatar': 'avatar',

    # Links to put along the top menu
    'topmenu_links': [],

    # Relative paths to custom CSS/JS scripts (must be present in static files)
    'custom_css': None,
    'custom_js': None,

    #############
    # Side Menu #
    #############

    # Whether to display the side menu
    'show_sidebar': True,

    # Whether to aut expand the menu
    'navigation_expanded': True,

    # Hide these apps when generating side menu
    'hide_apps': [],

    # Hide these models when generating side menu
    'hide_models': [],

    # List of apps to base side menu ordering off of
    'order_with_respect_to': [],

    # Custom links to append to app groups, keyed on app name
    'custom_links': {},

    # Custom icons per model in the side menu See https://www.fontawesomecheatsheet.com/font-awesome-cheatsheet-5x/
    # for a list of icon classes
    'icons': {
        'auth.user': 'fa-user',
    }
}


def get_settings():
    jazzmin_settings = copy.deepcopy(DEFAULT_SETTINGS)
    user_settings = {x: y for x, y in getattr(settings, 'JAZZMIN_SETTINGS', {}).items() if y is not None}
    jazzmin_settings.update(user_settings)

    if jazzmin_settings['search_model']:
        jazzmin_settings['search_url'] = get_admin_url(jazzmin_settings['search_model'].lower())
        jazzmin_settings['search_name'] = jazzmin_settings['search_model'].split('.')[-1] + 's'

    for link in jazzmin_settings.get('topmenu_links', []):
        if 'url' in link:
            link['url'] = get_custom_url(link['url'])
        elif 'model' in link:
            link['name'] = get_model_meta(link['model']).verbose_name_plural.title()
            link['url'] = get_admin_url(link['model'])
        elif 'app' in link:
            link['name'] = link['app'].title()
            link['app_children'] = get_app_admin_urls(link['app'])

    jazzmin_settings['hide_apps'] = [x.lower() for x in jazzmin_settings['hide_apps']]
    jazzmin_settings['hide_models'] = [x.lower() for x in jazzmin_settings['hide_models']]

    return jazzmin_settings



from __future__ import unicode_literals

from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _

__all__ = ['JazzminConfig']


class JazzminConfig(AppConfig):
    name = 'jazzmin'
    label = 'jazzmin'
    verbose_name = _('Jazzmin')
